@@ -37,20 +37,22 @@ import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import javax.xml.XMLConstants;
+import org.apache.http.HttpEntity;
+import org.apache.http.HttpResponse;
+import org.apache.http.HttpStatus;
+import org.apache.http.client.ClientProtocolException;
+import org.apache.http.client.HttpRequestRetryHandler;
+import org.apache.http.client.methods.HttpGet;
+import org.apache.http.impl.client.DefaultHttpClient;
+import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
+import org.apache.http.util.EntityUtils;
 
-import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
-import org.apache.commons.httpclient.HttpClient;
-import org.apache.commons.httpclient.HttpException;
-import org.apache.commons.httpclient.HttpStatus;
-import org.apache.commons.httpclient.methods.GetMethod;
-import org.apache.commons.httpclient.params.HttpMethodParams;
-import org.apache.maven.doxia.logging.Log;
 import org.apache.maven.doxia.macro.MacroExecutionException;
 import org.apache.maven.doxia.markup.XmlMarkup;
 import org.apache.maven.doxia.sink.Sink;
 import org.apache.maven.doxia.sink.SinkEventAttributeSet;
 import org.apache.maven.doxia.util.HtmlTools;
+import org.apache.maven.doxia.util.XmlValidator;
 
 import org.codehaus.plexus.util.FileUtils;
 import org.codehaus.plexus.util.IOUtil;
@@ -62,18 +64,12 @@ import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
 import org.xml.sax.EntityResolver;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
-import org.xml.sax.SAXNotRecognizedException;
-import org.xml.sax.SAXNotSupportedException;
-import org.xml.sax.SAXParseException;
-import org.xml.sax.XMLReader;
-import org.xml.sax.helpers.DefaultHandler;
-import org.xml.sax.helpers.XMLReaderFactory;
 
 /**
  * An abstract class that defines some convenience methods for <code>XML</code> parsers.
  *
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
- * @version $Id: AbstractXmlParser.java 807164 2009-08-24 11:59:25Z vsiveton $
+ * @version $Id: AbstractXmlParser.java 1035744 2010-11-16 18:48:53Z dennisl $
  * @since 1.0
  */
 public abstract class AbstractXmlParser
@@ -98,15 +94,6 @@ public abstract class AbstractXmlParser
     private static final Pattern PATTERN_ENTITY_2 =
         Pattern.compile( ENTITY_START + "(\\s)+([^>|^\\s]+)(\\s)+\"(\\s)*(&(#x?[0-9a-fA-F]{1,5};)*)(\\s)*\"(\\s)*>" );
 
-    /**
-     * Doctype pattern i.e. ".*<!DOCTYPE([^>]*)>.*"
-     * see <a href="http://www.w3.org/TR/REC-xml/#NT-doctypedecl">http://www.w3.org/TR/REC-xml/#NT-doctypedecl</a>.
-     */
-    private static final Pattern PATTERN_DOCTYPE = Pattern.compile( ".*" + DOCTYPE_START + "([^>]*)>.*" );
-
-    /** Tag pattern as defined in http://www.w3.org/TR/REC-xml/#NT-Name */
-    private static final Pattern PATTERN_TAG = Pattern.compile( ".*<([A-Za-z][A-Za-z0-9:_.-]*)([^>]*)>.*" );
-
     private boolean ignorableWhitespace;
 
     private boolean collapsibleWhitespace;
@@ -115,10 +102,7 @@ public abstract class AbstractXmlParser
 
     private Map entities;
 
-    private boolean validate = true;
-
-    /** lazy xmlReader to validate xml content*/
-    private XMLReader xmlReader;
+    private boolean validate = false;
 
     /** {@inheritDoc} */
     public void parse( Reader source, Sink sink )
@@ -139,7 +123,7 @@ public abstract class AbstractXmlParser
                 throw new ParseException( "Error reading the model: " + e.getMessage(), e );
             }
 
-            validate( content );
+            new XmlValidator( getLog() ).validate( content );
 
             source = new StringReader( content );
         }
@@ -417,7 +401,7 @@ public abstract class AbstractXmlParser
      * @param type the tag event type. This should be one of HtmlMarkup.TAG_TYPE_SIMPLE,
      * HtmlMarkup.TAG_TYPE_START, HtmlMarkup.TAG_TYPE_END or HtmlMarkup.ENTITY_TYPE.
      * It will be passed as the first argument of the required parameters to the Sink
-     * {@link org.apache.maven.doxia.sink.Sink#unknown(String, Object[], SinkEventAttributes)}
+     * {@link org.apache.maven.doxia.sink.Sink#unknown(String, Object[], org.apache.maven.doxia.sink.SinkEventAttributes)}
      * method.
      */
     protected void handleUnknown( XmlPullParser parser, Sink sink, int type )
@@ -593,90 +577,6 @@ public abstract class AbstractXmlParser
     // Private methods
     // ----------------------------------------------------------------------
 
-    /**
-     * Validate an XML content with SAX.
-     *
-     * @param content a not null xml content
-     * @throws ParseException if any.
-     */
-    private void validate( String content )
-        throws ParseException
-    {
-        try
-        {
-            // 1 if there's a doctype
-            boolean hasDoctype = false;
-            Matcher matcher = PATTERN_DOCTYPE.matcher( content );
-            if ( matcher.find() )
-            {
-                hasDoctype = true;
-            }
-
-            // 2 check for an xmlns instance
-            boolean hasXsd = false;
-            matcher = PATTERN_TAG.matcher( content );
-            if ( matcher.find() )
-            {
-                String value = matcher.group( 2 );
-
-                if ( value.indexOf( XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI ) != -1 )
-                {
-                    hasXsd = true;
-                }
-            }
-
-            // 3 validate content if doctype or xsd
-            if ( hasDoctype || hasXsd )
-            {
-                if ( getLog().isDebugEnabled() )
-                {
-                    getLog().debug( "Validating the content..." );
-                }
-                getXmlReader( hasXsd && hasDoctype ).parse( new InputSource( new StringReader( content ) ) );
-            }
-        }
-        catch ( IOException e )
-        {
-            throw new ParseException( "Error validating the model: " + e.getMessage(), e );
-        }
-        catch ( SAXNotRecognizedException e )
-        {
-            throw new ParseException( "Error validating the model: " + e.getMessage(), e );
-        }
-        catch ( SAXNotSupportedException e )
-        {
-            throw new ParseException( "Error validating the model: " + e.getMessage(), e );
-        }
-        catch ( SAXException e )
-        {
-            throw new ParseException( "Error validating the model: " + e.getMessage(), e );
-        }
-    }
-
-    /**
-     * @param hasDtdAndXsd to flag the <code>ErrorHandler</code>.
-     * @return an xmlReader instance.
-     * @throws SAXException if any
-     */
-    private XMLReader getXmlReader( boolean hasDtdAndXsd )
-        throws SAXException
-    {
-        if ( xmlReader == null )
-        {
-            MessagesErrorHandler errorHandler = new MessagesErrorHandler( getLog() );
-
-            xmlReader = XMLReaderFactory.createXMLReader( "org.apache.xerces.parsers.SAXParser" );
-            xmlReader.setFeature( "http://xml.org/sax/features/validation", true );
-            xmlReader.setFeature( "http://apache.org/xml/features/validation/schema", true );
-            xmlReader.setErrorHandler( errorHandler );
-            xmlReader.setEntityResolver( new CachedFileEntityResolver() );
-        }
-
-        ( (MessagesErrorHandler) xmlReader.getErrorHandler() ).setHasDtdAndXsd( hasDtdAndXsd );
-
-        return xmlReader;
-    }
-
     /**
      * Add an entity given by <code>entityName</code> and <code>entityValue</code> to {@link #entities}.
      * <br/>
@@ -796,125 +696,6 @@ public abstract class AbstractXmlParser
         }
     }
 
-    /**
-     * Convenience class to beautify <code>SAXParseException</code> messages.
-     */
-    static class MessagesErrorHandler
-        extends DefaultHandler
-    {
-        private static final int TYPE_UNKNOWN = 0;
-
-        private static final int TYPE_WARNING = 1;
-
-        private static final int TYPE_ERROR = 2;
-
-        private static final int TYPE_FATAL = 3;
-
-        /** @see org/apache/xerces/impl/msg/XMLMessages.properties#MSG_ELEMENT_NOT_DECLARED */
-        private static final Pattern ELEMENT_TYPE_PATTERN =
-            Pattern.compile( "Element type \".*\" must be declared.", Pattern.DOTALL );
-
-        private final Log log;
-
-        private boolean hasDtdAndXsd;
-
-        public MessagesErrorHandler( Log log )
-        {
-            this.log = log;
-        }
-
-        /**
-         * @param hasDtdAndXsd the hasDtdAndXsd to set
-         */
-        protected void setHasDtdAndXsd( boolean hasDtdAndXsd )
-        {
-            this.hasDtdAndXsd = hasDtdAndXsd;
-        }
-
-        /** {@inheritDoc} */
-        public void warning( SAXParseException e )
-            throws SAXException
-        {
-            processException( TYPE_WARNING, e );
-        }
-
-        /** {@inheritDoc} */
-        public void error( SAXParseException e )
-            throws SAXException
-        {
-            // Workaround for Xerces complaints when an XML with XSD needs also a <!DOCTYPE []> to specify entities
-            // like &nbsp;
-            // See http://xsd.stylusstudio.com/2001Nov/post08021.htm
-            if ( !hasDtdAndXsd )
-            {
-                processException( TYPE_ERROR, e );
-                return;
-            }
-
-            Matcher m = ELEMENT_TYPE_PATTERN.matcher( e.getMessage() );
-            if ( !m.find() )
-            {
-                processException( TYPE_ERROR, e );
-            }
-        }
-
-        /** {@inheritDoc} */
-        public void fatalError( SAXParseException e )
-            throws SAXException
-        {
-            processException( TYPE_FATAL, e );
-        }
-
-        private void processException( int type, SAXParseException e )
-            throws SAXException
-        {
-            StringBuffer message = new StringBuffer();
-
-            switch ( type )
-            {
-                case TYPE_WARNING:
-                    message.append( "Warning:" );
-                    break;
-
-                case TYPE_ERROR:
-                    message.append( "Error:" );
-                    break;
-
-                case TYPE_FATAL:
-                    message.append( "Fatal error:" );
-                    break;
-
-                case TYPE_UNKNOWN:
-                default:
-                    message.append( "Unknown:" );
-                    break;
-            }
-
-            message.append( EOL );
-            message.append( "  Public ID: " + e.getPublicId() ).append( EOL );
-            message.append( "  System ID: " + e.getSystemId() ).append( EOL );
-            message.append( "  Line number: " + e.getLineNumber() ).append( EOL );
-            message.append( "  Column number: " + e.getColumnNumber() ).append( EOL );
-            message.append( "  Message: " + e.getMessage() ).append( EOL );
-
-            switch ( type )
-            {
-                case TYPE_WARNING:
-                    if ( log.isWarnEnabled() )
-                    {
-                        log.warn( message.toString() );
-                    }
-                    break;
-
-                case TYPE_UNKNOWN:
-                case TYPE_ERROR:
-                case TYPE_FATAL:
-                default:
-                    throw new SAXException( message.toString() );
-            }
-        }
-    }
-
     /**
      * Implementation of the callback mechanism <code>EntityResolver</code>.
      * Using a mechanism of cached files in temp dir to improve performance when using the <code>XMLReader</code>.
@@ -943,14 +724,16 @@ public abstract class AbstractXmlParser
                     {
                         // Doxia XSDs are included in the jars, so try to find the resource systemName from
                         // the classpath...
-                        URL url = getClass().getResource( "/" + systemName );
+                        String resource = "/" + systemName;
+                        URL url = getClass().getResource( resource );
                         if ( url != null )
                         {
                             res = toByteArray( url );
                         }
                         else
                         {
-                            throw new SAXException( "Could not find the SYSTEM entity: " + systemId );
+                            throw new SAXException( "Could not find the SYSTEM entity: " + systemId
+                            + " because '" + resource + "' is not available of the classpath." );
                         }
                     }
                     else
@@ -980,7 +763,7 @@ public abstract class AbstractXmlParser
         /**
          * If url is not an http/https urls, call {@link IOUtil#toByteArray(java.io.InputStream)} to get the url
          * content.
-         * Otherwise, call {@link GetMethod#getResponseBody()} from HttpClient to get the http content.
+         * Otherwise, use HttpClient to get the http content.
          * Wrap all internal exceptions to throw SAXException.
          *
          * @param url not null
@@ -1013,25 +796,33 @@ public abstract class AbstractXmlParser
             }
 
             // it is an HTTP url, using HttpClient...
-            HttpClient client = new HttpClient();
-            GetMethod method = new GetMethod( url.toString() );
+            DefaultHttpClient client = new DefaultHttpClient();
+            HttpGet method = new HttpGet( url.toString() );
+            // Set a user-agent that doesn't contain the word "java", otherwise it will be blocked by the W3C
+            // The default user-agent is "Apache-HttpClient/4.0.2 (java 1.5)"
+            method.setHeader( "user-agent", "Apache-Doxia/1.1.4" );
 
-            method.getParams().setParameter( HttpMethodParams.RETRY_HANDLER,
-                                             new DefaultHttpMethodRetryHandler( 3, false ) );
+            HttpRequestRetryHandler retryHandler = new DefaultHttpRequestRetryHandler( 3, false );
+            client.setHttpRequestRetryHandler(retryHandler);
 
+            HttpEntity entity = null;
             try
             {
-                int statusCode = client.executeMethod( method );
+                HttpResponse response = client.execute( method );
+                int statusCode = response.getStatusLine().getStatusCode();
                 if ( statusCode != HttpStatus.SC_OK )
                 {
-                    throw new IOException( "Method failed: " + method.getStatusLine() );
+                    throw new IOException( "The status code when accessing the URL '" + url.toString() + "' was "
+                        + statusCode + ", which is not allowed. The server gave this reason for the failure '"
+                        + response.getStatusLine().getReasonPhrase() + "'.");
                 }
 
-                return method.getResponseBody();
+                entity = response.getEntity();
+                return EntityUtils.toByteArray( entity );
             }
-            catch ( HttpException e )
+            catch ( ClientProtocolException e )
             {
-                throw new SAXException( "HttpException: Fatal protocol violation: " + e.getMessage(), e );
+                throw new SAXException( "ClientProtocolException: Fatal protocol violation: " + e.getMessage(), e );
             }
             catch ( IOException e )
             {
@@ -1039,7 +830,16 @@ public abstract class AbstractXmlParser
             }
             finally
             {
-                method.releaseConnection();
+                if ( entity != null ) {
+                    try
+                    {
+                        entity.consumeContent();
+                    }
+                    catch ( IOException e )
+                    {
+                        // Ignore
+                    }
+                }
             }
         }
 
