@@ -44,7 +44,7 @@ import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
  *
  * @author <a href="mailto:jason@maven.org">Jason van Zyl</a>
  * @author ltheussl
- * @version $Id: XhtmlBaseParser.java 807164 2009-08-24 11:59:25Z vsiveton $
+ * @version $Id: XhtmlBaseParser.java 892639 2009-12-20 18:49:58Z ltheussl $
  * @since 1.1
  */
 public class XhtmlBaseParser
@@ -925,7 +925,7 @@ public class XhtmlBaseParser
         String border = parser.getAttributeValue( null, Attribute.BORDER.toString() );
         boolean grid = true;
 
-        if ( "0".equals( border ) )
+        if ( border == null || "0".equals( border ) )
         {
             grid = false;
         }
