@@ -32,7 +32,7 @@ import org.codehaus.plexus.PlexusTestCase;
  * Test for XhtmlBaseParser.
  *
  * @author ltheussl
- * @version $Id: XhtmlBaseParserTest.java 806492 2009-08-21 10:57:56Z ltheussl $
+ * @version $Id: XhtmlBaseParserTest.java 892639 2009-12-20 18:49:58Z ltheussl $
  * @since 1.1
  */
 public class XhtmlBaseParserTest
@@ -551,7 +551,12 @@ public class XhtmlBaseParserTest
         Iterator it = sink.getEventList().iterator();
 
         assertEquals( "table", ( (SinkEventElement) it.next() ).getName() );
-        assertEquals( "tableRows", ( (SinkEventElement) it.next() ).getName() );
+
+        // DOXIA-374
+        SinkEventElement el = (SinkEventElement) it.next();
+        assertEquals( "tableRows", el.getName() );
+        assertFalse( ( (Boolean) el.getArgs()[1] ).booleanValue() );
+
         assertEquals( "tableCaption", ( (SinkEventElement) it.next() ).getName() );
         assertEquals( "tableCaption_", ( (SinkEventElement) it.next() ).getName() );
         assertEquals( "tableRow", ( (SinkEventElement) it.next() ).getName() );
