@@ -28,7 +28,7 @@ import org.codehaus.plexus.PlexusTestCase;
  * Test for XhtmlBaseSink.
  *
  * @author ltheussl
- * @version $Id: XhtmlBaseSinkTest.java 806515 2009-08-21 11:58:39Z vsiveton $
+ * @version $Id: XhtmlBaseSinkTest.java 926049 2010-03-22 12:40:16Z ltheussl $
  * @since 1.1
  */
 public class XhtmlBaseSinkTest
@@ -629,7 +629,7 @@ public class XhtmlBaseSinkTest
             sink.close();
         }
 
-        assertEquals( "<table align=\"center\" border=\"0\" class=\"bodyTable\">", writer.toString() );
+        assertEquals( "<table border=\"0\" class=\"bodyTable\">", writer.toString() );
     }
 
     /**
@@ -714,7 +714,7 @@ public class XhtmlBaseSinkTest
             sink.close();
         }
 
-        assertEquals( "<table align=\"center\" border=\"0\" class=\"bodyTable\">" +
+        assertEquals( "<table border=\"0\" class=\"bodyTable\">" +
                 "<caption style=\"bold\">caption</caption></table>", writer.toString() );
     }
 
