@@ -49,7 +49,7 @@ import java.util.TreeSet;
  * <br/>
  * Based on the <a href="http://www.xmlmind.com/aptconvert.html">APTconvert</a> project.
  *
- * @version $Id: AptParser.java 812399 2009-09-08 08:57:35Z ltheussl $
+ * @version $Id: AptParser.java 946932 2010-05-21 08:36:30Z ltheussl $
  * @since 1.0
  * @plexus.component role="org.apache.maven.doxia.parser.Parser" role-hint="apt"
  */
@@ -217,6 +217,13 @@ public class AptParser
             // Lookahead block.
             nextBlock( /*first*/true );
 
+            // traverse comments
+            while ( ( block != null ) && ( block.getType() == COMMENT_BLOCK ) )
+            {
+                block.traverse();
+                nextBlock( /*first*/true );
+            }
+
             traverseHead();
 
             traverseBody();
@@ -2909,6 +2916,8 @@ public class AptParser
             aptParser.enableLogging( getLog() );
             parameters.put( "parser", aptParser );
 
+            // getBasedir() does not work in multi-module builds, see DOXIA-373
+            // the basedir should be injected from here, see DOXIA-224
             MacroRequest request = new MacroRequest( parameters, getBasedir() );
             try
             {
