@@ -38,7 +38,7 @@ import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
  * and emit events into the specified doxia Sink.
  *
  * @author <a href="mailto:jason@maven.org">Jason van Zyl</a>
- * @version $Id: DocBookParser.java 784737 2009-06-15 11:46:08Z ltheussl $
+ * @version $Id: DocBookParser.java 807168 2009-08-24 12:04:46Z vsiveton $
  * @since 1.0
  * @plexus.component role="org.apache.maven.doxia.parser.Parser" role-hint="docbook"
  */
@@ -171,13 +171,15 @@ public class DocBookParser
     /** {@inheritDoc} */
     protected void init()
     {
-        parent.clear();
-        trademark = 0;
-        level = 0;
-        isBold = false;
-        inHead = false;
-        ignore = false;
-        simpleTag = false;
+        super.init();
+
+        this.parent.clear();
+        this.trademark = 0;
+        this.level = 0;
+        this.isBold = false;
+        this.inHead = false;
+        this.ignore = false;
+        this.simpleTag = false;
     }
 
     // ----------------------------------------------------------------------
