@@ -48,7 +48,7 @@ import org.codehaus.plexus.util.StringUtils;
  * It uses the Docbook v4.4 DTD <a href="http://www.oasis-open.org/docbook/sgml/4.4/docbookx.dtd">
  * http://www.oasis-open.org/docbook/sgml/4.4/docbookx.dtd</a>.
  *
- * @version $Id: DocBookSink.java 785531 2009-06-17 09:47:59Z ltheussl $
+ * @version $Id: DocBookSink.java 807168 2009-08-24 12:04:46Z vsiveton $
  * @since 1.0
  */
 public class DocBookSink
@@ -506,8 +506,16 @@ public class DocBookSink
 
     /**
      * Reset all variables.
+     *
+     * @deprecated since 1.1.2, use {@link #init()} instead of.
      */
     protected void resetState()
+    {
+       init();
+    }
+
+    /** {@inheritDoc} */
+    protected void init()
     {
         hasTitle = false;
         authorDateFlag = false;
@@ -532,7 +540,7 @@ public class DocBookSink
      */
     public void head()
     {
-        resetState();
+        init();
 
         MutableAttributeSet att = writeXmlHeader( "article" );
 
@@ -672,7 +680,7 @@ public class DocBookSink
     {
         writeEndTag( SimplifiedDocbookMarkup.ARTICLE_TAG );
         out.flush();
-        resetState();
+        init();
     }
 
     /**
