@@ -56,7 +56,7 @@ import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
  *
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse</a>
  * @author ltheussl
- * @version $Id: FmlParser.java 784725 2009-06-15 10:57:11Z vsiveton $
+ * @version $Id: FmlParser.java 807169 2009-08-24 12:05:48Z vsiveton $
  * @since 1.0
  * @plexus.component role="org.apache.maven.doxia.parser.Parser" role-hint="fml"
  */
@@ -93,6 +93,10 @@ public class FmlParser
     public void parse( Reader source, Sink sink )
         throws ParseException
     {
+        this.faqs = null;
+        this.sourceContent = null;
+        init();
+
         try
         {
             StringWriter contentWriter = new StringWriter();
@@ -121,8 +125,12 @@ public class FmlParser
         }
         finally
         {
-
             logWarnings();
+
+            this.faqs = null;
+            this.sourceContent = null;
+            setSecondParsing( false );
+            init();
         }
     }
 
@@ -424,6 +432,19 @@ public class FmlParser
         }
     }
 
+    /** {@inheritDoc} */
+    protected void init()
+    {
+        super.init();
+
+        this.currentFaq = null;
+        this.currentPart = null;
+        this.buffer = null;
+        this.warnMessages = null;
+        this.macroName = null;
+        this.macroParameters = null;
+    }
+
     /**
      * TODO import from XdocParser, probably need to be generic.
      *
