@@ -25,6 +25,7 @@ import java.io.Writer;
 import java.util.Calendar;
 import java.util.Date;
 import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
 import java.util.ResourceBundle;
@@ -63,11 +64,33 @@ import org.codehaus.plexus.util.StringUtils;
  * of them are used in this sink (i.e. author, confidential, date and title), the others are ignored.
  *
  * @author ltheussl
- * @version $Id: FoAggregateSink.java 783586 2009-06-11 00:11:20Z vsiveton $
+ * @version $Id: FoAggregateSink.java 811802 2009-09-06 10:49:37Z vsiveton $
  * @since 1.1
  */
-public class FoAggregateSink extends FoSink
+public class FoAggregateSink
+    extends FoSink
 {
+    /**
+     * No Table Of Content.
+     * @see #setDocumentModel(DocumentModel, int)
+     */
+    public static int TOC_NONE = 0;
+
+    /**
+     * Table Of Content at the start of the document.
+     * @see #setDocumentModel(DocumentModel, int)
+     */
+    public static int TOC_START = 1;
+
+    /**
+     * Table Of Content at the end of the document.
+     * @see #setDocumentModel(DocumentModel, int)
+     */
+    public static int TOC_END = 2;
+
+    // TODO: make configurable
+    private static final String COVER_HEADER_HEIGHT = "1.5in";
+
     /** The document model to be used by this sink. */
     private DocumentModel docModel;
 
@@ -83,12 +106,12 @@ public class FoAggregateSink extends FoSink
     /** Content in head is ignored in aggregated documents. */
     private boolean ignoreText;
 
+    /** Current position of the TOC, see {@link #TOC_POSITION} */
+    private int tocPosition;
+
     /** Used to get the current position in the TOC. */
     private final Stack tocStack = new Stack();
 
-    // TODO: make configurable
-    private static final String COVER_HEADER_HEIGHT = "1.5in";
-
     /**
      * Constructor.
      *
@@ -108,6 +131,8 @@ public class FoAggregateSink extends FoSink
     /** {@inheritDoc} */
     public void head( SinkEventAttributes attributes )
     {
+        init();
+
         ignoreText = true;
     }
 
@@ -225,7 +250,6 @@ public class FoAggregateSink extends FoSink
         }
     }
 
-
     /**
      * Sets the name of the current source document, relative to the source root.
      * Used to resolve links to other source documents.
@@ -240,12 +264,57 @@ public class FoAggregateSink extends FoSink
     /**
      * Sets the DocumentModel to be used by this sink. The DocumentModel provides all the meta-information
      * required to render a document, eg settings for the cover page, table of contents, etc.
+     * <br/>
+     * By default, a TOC will be added at the beginning of the document.
      *
      * @param model the DocumentModel.
+     * @see #setDocumentModel(DocumentModel, String)
+     * @see #TOC_START
      */
     public void setDocumentModel( DocumentModel model )
+    {
+        setDocumentModel( model, TOC_START );
+    }
+
+    /**
+     * Sets the DocumentModel to be used by this sink. The DocumentModel provides all the meta-information
+     * required to render a document, eg settings for the cover page, table of contents, etc.
+     *
+     * @param model the DocumentModel, could be null.
+     * @param tocPos should be one of these values: {@link #TOC_NONE}, {@link #TOC_START} and {@link #TOC_END}.
+     * @since 1.1.2
+     */
+    public void setDocumentModel( DocumentModel model, int tocPos )
     {
         this.docModel = model;
+        if ( !( tocPos == TOC_NONE || tocPos == TOC_START || tocPos == TOC_END ) )
+        {
+            if ( getLog().isDebugEnabled() )
+            {
+                getLog().debug( "Unrecognized value for tocPosition: " + tocPos + ", using no toc." );
+            }
+            tocPos = TOC_NONE;
+        }
+        this.tocPosition = tocPos;
+
+        if ( this.docModel != null && this.docModel.getToc() != null && this.tocPosition != TOC_NONE )
+        {
+            DocumentTOCItem tocItem = new DocumentTOCItem();
+            tocItem.setName( this.docModel.getToc().getName() );
+            tocItem.setRef( "./toc" );
+            List items = new LinkedList();
+            if ( this.tocPosition == TOC_START )
+            {
+                items.add( tocItem );
+            }
+            items.addAll( this.docModel.getToc().getItems() );
+            if ( this.tocPosition == TOC_END )
+            {
+                items.add( tocItem );
+            }
+
+            this.docModel.getToc().setItems( items );
+        }
     }
 
     /**
@@ -271,9 +340,14 @@ public class FoAggregateSink extends FoSink
             idName = "./" + idName;
         }
 
-        if ( idName.indexOf( ".", 2 ) != -1 )
+        if ( idName.substring( 2 ).lastIndexOf( "." ) != -1 )
         {
-            idName = idName.substring( 0, idName.indexOf( ".", 2 ) );
+            idName = idName.substring( 0, idName.lastIndexOf( "." ) );
+        }
+
+        while ( idName.indexOf( "//" ) != -1 )
+        {
+            idName = StringUtils.replace( idName, "//", "/" );
         }
 
         return idName;
@@ -314,7 +388,7 @@ public class FoAggregateSink extends FoSink
     }
 
     /** {@inheritDoc} */
-    public void anchor( String name, SinkEventAttributes attributes  )
+    public void anchor( String name, SinkEventAttributes attributes )
     {
         if ( name == null )
         {
@@ -342,13 +416,13 @@ public class FoAggregateSink extends FoSink
     }
 
     /** {@inheritDoc} */
-    public void link( String name  )
+    public void link( String name )
     {
         link( name, null );
     }
 
     /** {@inheritDoc} */
-    public void link( String name, SinkEventAttributes attributes  )
+    public void link( String name, SinkEventAttributes attributes )
     {
         if ( name == null )
         {
@@ -360,11 +434,17 @@ public class FoAggregateSink extends FoSink
             // external links
             writeStartTag( BASIC_LINK_TAG, "external-destination", HtmlTools.escapeHTML( name ) );
             writeStartTag( INLINE_TAG, "href.external" );
+            return;
         }
-        else if ( DoxiaUtils.isInternalLink( name ) )
+
+        while ( name.indexOf( "//" ) != -1 )
+        {
+            name = StringUtils.replace( name, "//", "/" );
+        }
+
+        if ( DoxiaUtils.isInternalLink( name ) )
         {
             // internal link (ie anchor is in the same source document)
-
             String anchor = name.substring( 1 );
 
             if ( !DoxiaUtils.isValidId( anchor ) )
@@ -414,7 +494,7 @@ public class FoAggregateSink extends FoSink
                 return;
             }
 
-            anchor = chopExtension ( anchor );
+            anchor = chopExtension( anchor );
 
             String base = docName.substring( 0, docName.lastIndexOf( "/" ) );
             anchor = base + "/" + anchor;
@@ -438,13 +518,22 @@ public class FoAggregateSink extends FoSink
                 base = base.substring( 0, base.lastIndexOf( "/" ) );
 
                 anchor = anchor.substring( 3 );
+
+                if ( base.lastIndexOf( "/" ) == -1 )
+                {
+                    while ( anchor.startsWith( "../" ) )
+                    {
+                        anchor = anchor.substring( 3 );
+                    }
+                    break;
+                }
             }
         }
 
         return base + "/" + anchor;
     }
 
-    private  String chopExtension( String name )
+    private String chopExtension( String name )
     {
         String anchor = name;
 
@@ -460,13 +549,14 @@ public class FoAggregateSink extends FoSink
 
                 if ( dot2 != -1 )
                 {
-                    anchor = anchor.substring( 0, dot ) + "#"
-                        + HtmlTools.encodeId( anchor.substring( hash + 1, dot2 ) );
+                    anchor =
+                        anchor.substring( 0, dot ) + "#" + HtmlTools.encodeId( anchor.substring( hash + 1, dot2 ) );
                 }
                 else
                 {
-                    anchor = anchor.substring( 0, dot ) + "#"
-                        + HtmlTools.encodeId( anchor.substring( hash + 1, anchor.length() ) );
+                    anchor =
+                        anchor.substring( 0, dot ) + "#"
+                            + HtmlTools.encodeId( anchor.substring( hash + 1, anchor.length() ) );
                 }
             }
             else
@@ -759,7 +849,8 @@ public class FoAggregateSink extends FoSink
      */
     public void toc()
     {
-        if ( docModel == null || docModel.getToc() == null || docModel.getToc().getItems() == null )
+        if ( docModel == null || docModel.getToc() == null || docModel.getToc().getItems() == null
+            || this.tocPosition == TOC_NONE )
         {
             return;
         }
@@ -770,6 +861,7 @@ public class FoAggregateSink extends FoSink
         regionBefore( toc.getName() );
         regionAfter( getFooterText() );
         writeStartTag( FLOW_TAG, "flow-name", "xsl-region-body" );
+        writeStartTag( BLOCK_TAG, "id", "./toc" );
         chapterHeading( toc.getName(), false );
         writeln( "<fo:table table-layout=\"fixed\" width=\"100%\" >" );
         writeEmptyTag( TABLE_COLUMN_TAG, "column-width", "0.45in" );
@@ -782,6 +874,7 @@ public class FoAggregateSink extends FoSink
 
         writeEndTag( TABLE_BODY_TAG );
         writeEndTag( TABLE_TAG );
+        writeEndTag( BLOCK_TAG );
         writeEndTag( FLOW_TAG );
         writeEndTag( PAGE_SEQUENCE_TAG );
     }
@@ -1056,11 +1149,9 @@ public class FoAggregateSink extends FoSink
         writeSimpleTag( BLOCK_TAG );
         writeEndTag( TABLE_CELL_TAG );
 
-
         writeStartTag( TABLE_CELL_TAG, "number-columns-spanned", "2", "cover.border.left.bottom" );
         writeStartTag( BLOCK_TAG, "cover.subtitle" );
-        write( subtitle == null ? " v. " + ( version == null ? "" : version )
-                        : subtitle );
+        write( subtitle == null ? ( version == null ? "" : " v. " + version ) : subtitle );
         writeEndTag( BLOCK_TAG );
         writeStartTag( BLOCK_TAG, "cover.subtitle" );
         write( type == null ? "" : type );
@@ -1090,7 +1181,7 @@ public class FoAggregateSink extends FoSink
         writeEndTag( TABLE_ROW_TAG );
     }
 
-    private void writeCoverFooter( DocumentCover cover, DocumentMeta meta  )
+    private void writeCoverFooter( DocumentCover cover, DocumentMeta meta )
     {
         if ( cover == null && meta == null )
         {
@@ -1164,7 +1255,7 @@ public class FoAggregateSink extends FoSink
 
         if ( atts == null )
         {
-            return new SinkEventAttributeSet( new String[] {SinkEventAttributes.HEIGHT, COVER_HEADER_HEIGHT} );
+            return new SinkEventAttributeSet( new String[] { SinkEventAttributes.HEIGHT, COVER_HEADER_HEIGHT } );
         }
 
         // FOP dpi: 72
