@@ -36,7 +36,7 @@ import org.apache.maven.doxia.sink.SinkTestDocument;
 /**
  * <code>FO Sink</code> Test case.
  *
- * @version $Id: FoSinkTest.java 809309 2009-08-30 11:01:10Z vsiveton $
+ * @version $Id: FoSinkTest.java 946933 2010-05-21 08:39:07Z ltheussl $
  */
 public class FoSinkTest
     extends AbstractSinkTest
@@ -262,7 +262,7 @@ public class FoSinkTest
         String attribs = getConfig().getAttributeString( "list" );
         String itemAttribs = getConfig().getAttributeString( "list.item" );
         return EOL + EOL + "<fo:list-block" + attribs + ">" + EOL + "<fo:list-item" + itemAttribs + ">" + EOL
-            + "<fo:list-item-label>" + EOL + "<fo:block>i</fo:block>" + EOL + "</fo:list-item-label>" + EOL + EOL
+            + "<fo:list-item-label>" + EOL + "<fo:block>i.</fo:block>" + EOL + "</fo:list-item-label>" + EOL + EOL
             + "<fo:list-item-body" + itemAttribs + ">" + EOL + "<fo:block>" + item + "</fo:block>" + EOL
             + "</fo:list-item-body>" + EOL + "</fo:list-item>" + EOL + "</fo:list-block>" + EOL;
     }
