@@ -67,7 +67,7 @@ import org.codehaus.plexus.util.xml.XMLWriter;
  * @see <a href="http://www.lowagie.com/iText/tutorial/ch07.html">http://www.lowagie.com/iText/tutorial/ch07.html</a>
  *
  * @author <a href="mailto:vincent.siveton@gmail.com">Vincent Siveton</a>
- * @version $Id: ITextSink.java 807173 2009-08-24 12:14:25Z vsiveton $
+ * @version $Id: ITextSink.java 1003454 2010-10-01 09:46:51Z ltheussl $
  */
 public class ITextSink
     extends AbstractXmlSink
@@ -192,6 +192,12 @@ public class ITextSink
     {
         IOUtil.close( writer );
 
+        init();
+    }
+
+    /** {@inheritDoc} */
+    public void flush()
+    {
         if ( getLog().isWarnEnabled() && this.warnMessages != null )
         {
             for ( Iterator it = this.warnMessages.entrySet().iterator(); it.hasNext(); )
@@ -207,17 +213,9 @@ public class ITextSink
                     getLog().warn( msg );
                 }
             }
-
-            this.warnMessages = null;
         }
 
-        init();
-    }
-
-    /** {@inheritDoc} */
-    public void flush()
-    {
-        super.flush();
+        this.warnMessages = null;
     }
 
     // ----------------------------------------------------------------------
@@ -233,7 +231,7 @@ public class ITextSink
     /** {@inheritDoc} */
     public void head()
     {
-        init();
+        //init(); // why? this causes DOXIA-413
 
         actionContext.setAction( SinkActionContext.HEAD );
     }
@@ -1186,7 +1184,7 @@ public class ITextSink
                 try
                 {
                     nameFile = new File( System.getProperty( "itext.basedir" ), name );
-                    urlName = nameFile.toURL().toString();
+                    urlName = nameFile.toURI().toURL().toString();
                 }
                 catch ( MalformedURLException e )
                 {
