@@ -22,6 +22,7 @@ package org.apache.maven.doxia.module.latex;
 import org.apache.maven.doxia.sink.AbstractTextSink;
 import org.apache.maven.doxia.sink.Sink;
 import org.apache.maven.doxia.sink.SinkEventAttributes;
+import org.apache.maven.doxia.sink.SinkEventAttributeSet;
 import org.apache.maven.doxia.util.DoxiaUtils;
 import org.apache.maven.doxia.util.LineBreaker;
 
@@ -38,7 +39,7 @@ import java.util.Locale;
  * <br/>
  * <b>Note</b>: The encoding used is UTF-8.
  *
- * @version $Id: LatexSink.java 784533 2009-06-14 10:02:54Z vsiveton $
+ * @version $Id: LatexSink.java 807174 2009-08-24 12:15:08Z vsiveton $
  * @since 1.0
  */
 public class LatexSink
@@ -49,15 +50,15 @@ public class LatexSink
      *
      * This implies that <code>\\begin{document}</code>, <code>\\title{..}</code> will not be output.
      */
-    private boolean fragmentDocument;
+    private final boolean fragmentDocument;
 
     private boolean ignoreText;
 
-    private LineBreaker out;
+    private final LineBreaker out;
 
-    private String sinkCommands;
+    private final String sinkCommands;
 
-    private String preamble;
+    private final String preamble;
 
     private boolean titleFlag;
 
@@ -65,8 +66,6 @@ public class LatexSink
 
     private boolean verbatimFlag;
 
-    private boolean boxFlag;
-
     private boolean figureFlag;
 
     private boolean tableFlag;
@@ -93,9 +92,7 @@ public class LatexSink
      */
     protected LatexSink( Writer out )
     {
-        this.out = new LineBreaker( out );
-        this.sinkCommands = defaultSinkCommands();
-        this.preamble = defaultPreamble();
+        this( out, null, null );
     }
 
     /**
@@ -124,9 +121,21 @@ public class LatexSink
     protected LatexSink( Writer out, String sinkCommands, String preamble, boolean fragmentDocument )
     {
         this.out = new LineBreaker( out );
+
+        if ( sinkCommands == null )
+        {
+            sinkCommands = defaultSinkCommands();
+        }
+        if ( preamble == null )
+        {
+            preamble = defaultPreamble();
+        }
+
         this.sinkCommands = sinkCommands;
         this.preamble = preamble;
         this.fragmentDocument = fragmentDocument;
+
+        init();
     }
 
     // ----------------------------------------------------------------------
@@ -172,15 +181,13 @@ public class LatexSink
      */
     public void head()
     {
-        titleFlag = false;
-        numberedListNesting = 0;
-        verbatimFlag = false;
-        boxFlag = false;
-        figureFlag = false;
-        tableFlag = false;
-        gridFlag = false;
-        cellJustif = null;
-        cellCount = 0;
+        head( null );
+    }
+
+    /** {@inheritDoc} */
+    public void head( SinkEventAttributes attributes )
+    {
+        init();
 
         if ( !fragmentDocument )
         {
@@ -198,6 +205,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void body()
+    {
+        body( null );
+    }
+
+    /** {@inheritDoc} */
+    public void body( SinkEventAttributes attributes )
     {
         if ( titleFlag )
         {
@@ -230,6 +243,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void title()
+    {
+        title( null );
+    }
+
+    /** {@inheritDoc} */
+    public void title( SinkEventAttributes attributes )
     {
         if ( !fragmentDocument )
         {
@@ -261,6 +280,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void author()
+    {
+        author( null );
+    }
+
+    /** {@inheritDoc} */
+    public void author( SinkEventAttributes attributes )
     {
         if ( !fragmentDocument )
         {
@@ -291,6 +316,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void date()
+    {
+        date( null );
+    }
+
+    /** {@inheritDoc} */
+    public void date( SinkEventAttributes attributes )
     {
         if ( !fragmentDocument )
         {
@@ -317,6 +348,47 @@ public class LatexSink
         }
     }
 
+    /** {@inheritDoc} */
+    public void sectionTitle( int level, SinkEventAttributes attributes )
+    {
+        isTitle = true;
+    }
+
+    /** {@inheritDoc} */
+    public void sectionTitle_( int level )
+    {
+        String command = "";
+        switch ( level )
+        {
+            case SECTION_LEVEL_1:
+                command = "section";
+                break;
+            case SECTION_LEVEL_2:
+                command = "subsection";
+                break;
+            case SECTION_LEVEL_3:
+                command = "subsubsection";
+                break;
+            case SECTION_LEVEL_4:
+                command = "paragraph";
+                break;
+            case SECTION_LEVEL_5:
+                command = "subparagraph";
+                break;
+            default:
+                throw new IllegalArgumentException( "Not a section level: " + level );
+        }
+
+        isTitle = false;
+
+        if ( StringUtils.isNotEmpty( title ) )
+        {
+            markup( EOL + "\\" + command + "{" + title + "}" + EOL );
+
+            title = null;
+        }
+    }
+
     // ----------------------------------------------------------------------
     // Section Title 1
     // ----------------------------------------------------------------------
@@ -326,7 +398,7 @@ public class LatexSink
      */
     public void sectionTitle1()
     {
-        isTitle = true;
+        sectionTitle( SECTION_LEVEL_1, null );
     }
 
     /**
@@ -334,14 +406,7 @@ public class LatexSink
      */
     public void sectionTitle1_()
     {
-        isTitle = false;
-
-        if ( StringUtils.isNotEmpty( title ) )
-        {
-            markup( EOL + "\\section{" + title + "}" + EOL );
-
-            title = null;
-        }
+        sectionTitle_( SECTION_LEVEL_1 );
     }
 
     // ----------------------------------------------------------------------
@@ -353,7 +418,7 @@ public class LatexSink
      */
     public void sectionTitle2()
     {
-        isTitle = true;
+        sectionTitle( SECTION_LEVEL_2, null );
     }
 
     /**
@@ -361,14 +426,7 @@ public class LatexSink
      */
     public void sectionTitle2_()
     {
-        isTitle = false;
-
-        if ( StringUtils.isNotEmpty( title ) )
-        {
-            markup( EOL + "\\subsection{" + title + "}" + EOL );
-
-            title = null;
-        }
+        sectionTitle_( SECTION_LEVEL_2 );
     }
 
     // ----------------------------------------------------------------------
@@ -380,7 +438,7 @@ public class LatexSink
      */
     public void sectionTitle3()
     {
-        isTitle = true;
+        sectionTitle( SECTION_LEVEL_3, null );
     }
 
     /**
@@ -388,14 +446,7 @@ public class LatexSink
      */
     public void sectionTitle3_()
     {
-        isTitle = false;
-
-        if ( StringUtils.isNotEmpty( title ) )
-        {
-            markup( EOL + "\\subsubsection{" + title + "}" + EOL );
-
-            title = null;
-        }
+        sectionTitle_( SECTION_LEVEL_3 );
     }
 
     // ----------------------------------------------------------------------
@@ -407,7 +458,7 @@ public class LatexSink
      */
     public void sectionTitle4()
     {
-        isTitle = true;
+        sectionTitle( SECTION_LEVEL_4, null );
     }
 
     /**
@@ -415,14 +466,7 @@ public class LatexSink
      */
     public void sectionTitle4_()
     {
-        isTitle = false;
-
-        if ( StringUtils.isNotEmpty( title ) )
-        {
-            markup( EOL + "\\paragraph{" + title + "}" + EOL );
-
-            title = null;
-        }
+        sectionTitle_( SECTION_LEVEL_4 );
     }
 
     // ----------------------------------------------------------------------
@@ -434,7 +478,7 @@ public class LatexSink
      */
     public void sectionTitle5()
     {
-        isTitle = true;
+        sectionTitle( SECTION_LEVEL_5, null );
     }
 
     /**
@@ -442,14 +486,7 @@ public class LatexSink
      */
     public void sectionTitle5_()
     {
-        isTitle = false;
-
-        if ( StringUtils.isNotEmpty( title ) )
-        {
-            markup( EOL + "\\subparagraph{" + title + "}" + EOL );
-
-            title = null;
-        }
+        sectionTitle_( SECTION_LEVEL_5 );
     }
 
     // ----------------------------------------------------------------------
@@ -460,6 +497,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void list()
+    {
+        list( null );
+    }
+
+    /** {@inheritDoc} */
+    public void list( SinkEventAttributes attributes )
     {
         markup( EOL + "\\begin{itemize}" );
     }
@@ -476,6 +519,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void listItem()
+    {
+        listItem( null );
+    }
+
+    /** {@inheritDoc} */
+    public void listItem( SinkEventAttributes attributes )
     {
         markup( EOL + "\\item " );
     }
@@ -484,6 +533,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void numberedList( int numbering )
+    {
+        numberedList( numbering, null );
+    }
+
+    /** {@inheritDoc} */
+    public void numberedList( int numbering, SinkEventAttributes attributes )
     {
         ++numberedListNesting;
 
@@ -541,6 +596,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void numberedListItem()
+    {
+        numberedListItem( null );
+    }
+
+    /** {@inheritDoc} */
+    public void numberedListItem( SinkEventAttributes attributes )
     {
         markup( "\\item " );
     }
@@ -549,6 +610,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void definitionList()
+    {
+        definitionList( null );
+    }
+
+    /** {@inheritDoc} */
+    public void definitionList( SinkEventAttributes attributes )
     {
         markup( EOL + "\\begin{description}" );
     }
@@ -565,6 +632,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void definedTerm()
+    {
+        definedTerm( null );
+    }
+
+    /** {@inheritDoc} */
+    public void definedTerm( SinkEventAttributes attributes )
     {
         markup( EOL + "\\item[\\mbox{" );
     }
@@ -577,6 +650,42 @@ public class LatexSink
         markup( "}] " );
     }
 
+    /** {@inheritDoc} */
+    public void definitionListItem()
+    {
+        definitionListItem( null );
+    }
+
+    /** {@inheritDoc} */
+    public void definitionListItem( SinkEventAttributes attributes )
+    {
+        // nop
+    }
+
+    /** {@inheritDoc} */
+    public void definitionListItem_()
+    {
+        // nop
+    }
+
+    /** {@inheritDoc} */
+    public void definition()
+    {
+        definition( null );
+    }
+
+    /** {@inheritDoc} */
+    public void definition( SinkEventAttributes attributes )
+    {
+        // nop
+    }
+
+    /** {@inheritDoc} */
+    public void definition_()
+    {
+        // nop
+    }
+
     // ----------------------------------------------------------------------
     // Figure
     // ----------------------------------------------------------------------
@@ -585,6 +694,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void figure()
+    {
+        figure( null );
+    }
+
+    /** {@inheritDoc} */
+    public void figure( SinkEventAttributes attributes )
     {
         figureFlag = true;
         markup( EOL + "\\begin{figure}[htb]" + EOL );
@@ -604,13 +719,19 @@ public class LatexSink
      */
     public void figureGraphics( String name )
     {
-        if ( !name.toLowerCase( Locale.ENGLISH ).endsWith( ".eps" ) )
+        figureGraphics( name, null );
+    }
+
+    /** {@inheritDoc} */
+    public void figureGraphics( String src, SinkEventAttributes attributes )
+    {
+        if ( !src.toLowerCase( Locale.ENGLISH ).endsWith( ".eps" ) )
         {
             getLog().warn( "[Latex Sink] Found non-eps figure graphics!" );
         }
 
         markup( "\\begin{center}" + EOL );
-        markup( "\\includegraphics{" + name + "}" + EOL );
+        markup( "\\includegraphics{" + src + "}" + EOL );
         markup( "\\end{center}" + EOL );
     }
 
@@ -618,6 +739,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void figureCaption()
+    {
+        figureCaption( null );
+    }
+
+    /** {@inheritDoc} */
+    public void figureCaption( SinkEventAttributes attributes )
     {
         markup( "\\caption{" );
     }
@@ -638,6 +765,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void table()
+    {
+        table( null );
+    }
+
+    /** {@inheritDoc} */
+    public void table( SinkEventAttributes attributes )
     {
         tableFlag = true;
         markup( EOL + "\\begin{table}[htp]" + EOL );
@@ -711,6 +844,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void tableRow()
+    {
+        tableRow( null );
+    }
+
+    /** {@inheritDoc} */
+    public void tableRow( SinkEventAttributes attributes )
     {
         cellCount = 0;
     }
@@ -733,6 +872,21 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void tableCell()
+    {
+        tableCell( (SinkEventAttributes) null );
+    }
+
+    /** {@inheritDoc} */
+    public void tableCell( String width )
+    {
+        SinkEventAttributeSet att = new SinkEventAttributeSet();
+        att.addAttribute( javax.swing.text.html.HTML.Attribute.WIDTH, width );
+
+        tableCell( att );
+    }
+
+    /** {@inheritDoc} */
+    public void tableCell( SinkEventAttributes attributes )
     {
         tableCell( false );
     }
@@ -742,13 +896,29 @@ public class LatexSink
      */
     public void tableCell_()
     {
-        tableCell_( false );
+        markup( "\\end{tabular}" );
+        ++cellCount;
     }
 
     /**
      * {@inheritDoc}
      */
     public void tableHeaderCell()
+    {
+        tableCell( (SinkEventAttributes) null );
+    }
+
+    /** {@inheritDoc} */
+    public void tableHeaderCell( String width )
+    {
+        SinkEventAttributeSet att = new SinkEventAttributeSet();
+        att.addAttribute( javax.swing.text.html.HTML.Attribute.WIDTH, width );
+
+        tableHeaderCell( att );
+    }
+
+    /** {@inheritDoc} */
+    public void tableHeaderCell( SinkEventAttributes attributes )
     {
         tableCell( true );
     }
@@ -758,7 +928,7 @@ public class LatexSink
      */
     public void tableHeaderCell_()
     {
-        tableCell_( true );
+        tableCell_();
     }
 
     private boolean lastCellWasHeader = false;
@@ -794,21 +964,16 @@ public class LatexSink
         markup( "\\begin{tabular}[t]{" + justif + "}" );
     }
 
-    /**
-     * Ends a table cell.
-     *
-     * @param header True if this is a header cell.
-     */
-    private void tableCell_( boolean header )
-    {
-        markup( "\\end{tabular}" );
-        ++cellCount;
-    }
-
     /**
      * {@inheritDoc}
      */
     public void tableCaption()
+    {
+        tableCaption( null );
+    }
+
+    /** {@inheritDoc} */
+    public void tableCaption( SinkEventAttributes attributes )
     {
         markup( "\\caption{" );
     }
@@ -825,6 +990,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void paragraph()
+    {
+        paragraph( null );
+    }
+
+    /** {@inheritDoc} */
+    public void paragraph( SinkEventAttributes attributes )
     {
         markup( EOL + EOL );
     }
@@ -842,6 +1013,20 @@ public class LatexSink
      */
     public void verbatim( boolean boxed )
     {
+        verbatim( boxed ? SinkEventAttributeSet.BOXED : null );
+    }
+
+    /** {@inheritDoc} */
+    public void verbatim( SinkEventAttributes attributes )
+    {
+        boolean boxed = false;
+
+        if ( attributes != null && attributes.isDefined( SinkEventAttributes.DECORATION ) )
+        {
+            boxed = "boxed".equals(
+                attributes.getAttribute( SinkEventAttributes.DECORATION ) );
+        }
+
         markup( EOL + "\\begin{small}" + EOL );
 
         if ( boxed )
@@ -854,7 +1039,6 @@ public class LatexSink
         }
 
         verbatimFlag = true;
-        boxFlag = boxed;
     }
 
     /**
@@ -866,13 +1050,18 @@ public class LatexSink
         markup( "\\end{small}" + EOL );
 
         verbatimFlag = false;
-        boxFlag = false;
     }
 
     /**
      * {@inheritDoc}
      */
     public void horizontalRule()
+    {
+        horizontalRule( null );
+    }
+
+    /** {@inheritDoc} */
+    public void horizontalRule( SinkEventAttributes attributes )
     {
         markup( EOL + "\\begin{center}\\rule[0.5ex]{\\linewidth}{1pt}\\end{center}" + EOL );
     }
@@ -889,6 +1078,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void anchor( String name )
+    {
+        anchor( name, null );
+    }
+
+    /** {@inheritDoc} */
+    public void anchor( String name, SinkEventAttributes attributes )
     {
         markup( "\\hypertarget{" + name + "}{" );
     }
@@ -905,6 +1100,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void link( String name )
+    {
+        link( name, null );
+    }
+
+    /** {@inheritDoc} */
+    public void link( String name, SinkEventAttributes attributes )
     {
         // TODO: use \\url for simple links
         if ( DoxiaUtils.isExternalLink( name ) )
@@ -977,6 +1178,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void lineBreak()
+    {
+        lineBreak( null );
+    }
+
+    /** {@inheritDoc} */
+    public void lineBreak( SinkEventAttributes attributes )
     {
         markup( ( figureFlag || tableFlag || titleFlag || verbatimFlag ) ? EOL : "\\newline" + EOL );
     }
@@ -993,6 +1200,12 @@ public class LatexSink
      * {@inheritDoc}
      */
     public void text( String text )
+    {
+        text( text, null );
+    }
+
+    /** {@inheritDoc} */
+    public void text( String text, SinkEventAttributes attributes )
     {
         if ( ignoreText )
         {
@@ -1012,6 +1225,18 @@ public class LatexSink
         }
     }
 
+    /** {@inheritDoc} */
+    public void rawText( String text )
+    {
+        verbatimContent( text );
+    }
+
+    /** {@inheritDoc} */
+    public void comment( String comment )
+    {
+        rawText( EOL + "% " + comment );
+    }
+
     /**
      * {@inheritDoc}
      *
@@ -1140,6 +1365,8 @@ public class LatexSink
     public void close()
     {
         out.close();
+
+        init();
     }
 
     // ----------------------------------------------------------------------
@@ -1212,4 +1439,21 @@ public class LatexSink
         }
     }
 
+    /** {@inheritDoc} */
+    protected void init()
+    {
+        super.init();
+
+        this.ignoreText = false;
+        this.titleFlag = false;
+        this.numberedListNesting = 0;
+        this.verbatimFlag = false;
+        this.figureFlag = false;
+        this.tableFlag = false;
+        this.gridFlag = false;
+        this.cellJustif = null;
+        this.cellCount = 0;
+        this.isTitle = false;
+        this.title = null;
+    }
 }
