@@ -40,7 +40,7 @@ import org.codehaus.plexus.util.IOUtil;
 /**
  * @author <a href="mailto:jason@maven.org">Jason van Zyl</a>
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse</a>
- * @version $Id: XdocParserTest.java 806537 2009-08-21 13:06:21Z ltheussl $
+ * @version $Id: XdocParserTest.java 943628 2010-05-12 18:47:56Z ltheussl $
  * @since 1.0
  */
 public class XdocParserTest
@@ -334,6 +334,28 @@ public class XdocParserTest
         assertFalse( it.hasNext() );
     }
 
+    /** @throws Exception  */
+    public void testSourceContainingDTD()
+        throws Exception
+    {
+        String text = "<source><![CDATA[" +
+                          "<!DOCTYPE web-app PUBLIC " +
+                          "\"-//Sun Microsystems, Inc.//DTD Web Application 2.2//EN\"" +
+                          " \"http://java.sun.com/j2ee/dtds/web-app_2.2.dtd\">" +
+                      "]]></source>";
+
+        SinkEventTestingSink sink = new SinkEventTestingSink();
+
+        parser.parse( text, sink );
+
+        Iterator it = sink.getEventList().iterator();
+        assertEquals( "verbatim", ( (SinkEventElement) it.next() ).getName() );
+        assertEquals( "text", ( (SinkEventElement) it.next() ).getName() );
+        assertEquals( "verbatim_", ( (SinkEventElement) it.next() ).getName() );
+        assertFalse( it.hasNext() );
+
+    }
+
     /** @throws Exception  */
     public void testPreEOL()
         throws Exception
